#ifndef GLOBALS_H_
#define GLOBALS_H_

#include <stddef.h>
#include <sys/types.h>
#include <signal.h>
#include <pthread.h>
#include <libssh/server.h>

#define BUFSIZE 0x8000
#define MAXPATH 512
#define MAXARGS 31

struct connection_info_t
{
  struct connection_info_t *prev;
  struct connection_info_t *next;
  ssh_session   session;
  pthread_t     thread;
  char          buf[BUFSIZE];
  char         *cmdargs[MAXARGS+1];
  char          cmdline[BUFSIZE];
  char          pwd[MAXPATH];
  char          sockip[INET6_ADDRSTRLEN];
  int		dport;
  char          peerip[INET6_ADDRSTRLEN];
  int		sport;
};

struct globals_t
{
  char         *rsa_key;
  char         *dsa_key;
  char         *host_key;
  char         *bind_address;
  char         *bind_port;
  char         *pid_file;
  FILE         *flog;

  ssh_bind      sshbind;

  volatile size_t n_threads;
  volatile sig_atomic_t terminate;

  pthread_mutex_t mutex;

  struct connection_info_t *head;
  struct connection_info_t *tail;

  int           pid_fd;
  int           foreground;
  int           uid_set;
  int           gid_set;
  uid_t         uid;
  gid_t         gid;
};

extern struct globals_t globals;

void          init_globals( struct globals_t *g );
void          free_globals( struct globals_t *g );

#endif				/* GLOBALS_H_ */
